@@ -21,7 +21,7 @@ our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } ) ;
 
 our @EXPORT = qw() ;
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 # Preloaded methods go here.
 
@@ -274,7 +274,8 @@ sub insertall {
 				&& $self->{key} ne 'scalar' ;
 		$self->{sql}->{refto} = delete $self->{sql}->{item}
 				if $self->{key} eq 'perldata' ;
-		$self->{sql}->{chainedstring} = $ids{string} ;
+		$self->{sql}->{chainedstring} = $ids{string}
+				if $self->{sql}->{stringdata} ;
 
 		$self->sql( $dsn ) ;
 
@@ -388,6 +389,10 @@ Cleaned perldoc formatting issues
 
 Fixed bug: C<stringsplit()> converted 0 length strings to undefined
 
+=item 0.04	
+
+Fixed bug: C<insertall()> updating deleted records broke scalar chains
+
 =back
 
 
